import 'package:flutter_bloc/flutter_bloc.dart';

import 'delegate.dart';
import 'event.dart';
import 'state.dart';

class VideoBloc extends Bloc<VideoBlocEvent, VideoBlocState> {
  final VideoBlocDelegate delegate;

  VideoBloc({required this.delegate, required initialState})
      : super(initialState) {
    registerVideoListFetchEvent();
    registerAppendVideoListEvent();
    registerCleanVideoListEvent();
  }

  void registerVideoListFetchEvent() {
    on<VideoListFetchEvent>((event, emit) async {
      await delegate.fetchVideoList().then((list) {
        emit.call(VideoBlocState.success(videoList: list, finish: false));
        event.callback?.call();
      });
    });
  }

  void registerAppendVideoListEvent() {
    on<AppendVideoListEvent>((event, emit) async {
      await delegate.appendVideoList(state.videoList.length).then((list) {
        int length = state.videoList.length + event.length;
        state.videoList.addAll(list);
        emit.call(VideoBlocState.success(
            videoList: state.videoList,
            finish: state.videoList.length < length));
        event.callback?.call();
      });
    });
  }

  void registerCleanVideoListEvent() {
    on<CleanVideoListEvent>((event, emit) async {
      emit.call(VideoBlocState.init());
      event.callback?.call();
    });
  }
}
