import 'dart:async';
import 'dart:convert';
import 'dart:ui' as ui show Codec;
import 'dart:ui';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

import '../../constant/value/instance_container.dart';
import '../../delegate/data_service.dart';
import '../../delegate/load_balancer.dart';
import '../../util/instance_register.dart';

class Base64FileImageProvider extends ImageProvider<Base64FileImageProvider> {
  final String path;

  const Base64FileImageProvider(this.path);

  @override
  Future<Base64FileImageProvider> obtainKey(ImageConfiguration configuration) {
    return SynchronousFuture<Base64FileImageProvider>(this);
  }

  @override
  ImageStreamCompleter loadBuffer(
      Base64FileImageProvider key, DecoderBufferCallback decode) {
    return MultiFrameImageStreamCompleter(
      codec: _loadAsync(path),
      scale: 1.0,
    );
  }

  Future<ui.Codec> _loadAsync(String path) async {
    return _sendImageRequest().then((value) {
      return instantiateImageCodec(value);
    });
  }

  Future<Uint8List> _sendImageRequest() async {
    DataServiceDelegate dataServiceDelegate =
        InstanceRegister.resolve<DataServiceDelegate>(
            InstanceContainer.dataServiceDelegate);
    LoadBalancerDelegate loadBalancerDelegate =
        InstanceRegister.resolve<LoadBalancerDelegate>(
            InstanceContainer.loadBalancerDelegate);
    if (kIsWeb) {
      return dataServiceDelegate.getBinaryImageFile(path).then((base64Value) {
        return _generateImage(loadBalancerDelegate, base64Value);
      });
    } else {
      return _checkImageCache(dataServiceDelegate)
          .then((base64Value) =>
              _generateImage(loadBalancerDelegate, base64Value))
          .onError((error, stackTrace) {
        return dataServiceDelegate.getBinaryImageFile(path).then((base64Value) {
          _saveImageCache(dataServiceDelegate, base64Value);
          return _generateImage(loadBalancerDelegate, base64Value);
        });
      });
    }
  }

  Future<Uint8List> _generateImage(
      LoadBalancerDelegate loadBalancerDelegate, base64Value) {
    return loadBalancerDelegate.runIsolate<Uint8List, String>(
        base64Decode, base64Value);
  }

  Future<String> _checkImageCache(
      DataServiceDelegate dataServiceDelegate) async {
    return dataServiceDelegate.getBase64Cache(path);
  }

  void _saveImageCache(
      DataServiceDelegate dataServiceDelegate, String data) async {
    dataServiceDelegate.saveBase64Cache(path, data);
  }
}
