import 'package:carousel_slider/carousel_slider.dart';
import 'package:flutter/material.dart';

import '../delegate/carousel.dart';

class CarouselView extends StatelessWidget {
  final CarouselDelegate delegate;
  final bool centerMode;

  const CarouselView({super.key, required this.delegate}) : centerMode = false;

  const CarouselView.centerMode({super.key, required this.delegate})
      : centerMode = true;

  @override
  Widget build(BuildContext context) {
    return _getCarouselView(context);
  }

  _getCarouselView(BuildContext context) {
    if (centerMode) {
      return CarouselSlider(
        items: _getWidgetList(context),
        options: CarouselOptions(
            autoPlay: true,
            enlargeCenterPage: true,
            enableInfiniteScroll: true,
            viewportFraction: 0.8,
            enlargeStrategy: CenterPageEnlargeStrategy.scale,
            onPageChanged: (index, reason) {
              delegate.onSlide(index);
            }),
      );
    } else {
      return CarouselSlider(
        options: CarouselOptions(
          initialPage: 0,
          viewportFraction: 1.0,
          enlargeCenterPage: false,
          enableInfiniteScroll: true,
          autoPlay: true,
          onPageChanged: (index, reason) {
            delegate.onSlide(index);
          },
        ),
        items: _getWidgetList(context),
      );
    }
  }

  List<Widget> _getWidgetList(BuildContext context) {
    int itemCount = delegate.getClickCarouselItemSize();
    if (itemCount <= 0) {
      return [];
    }
    final widgets = <Widget>[];
    for (var i = 0; i < itemCount; i++) {
      widgets.add(delegate.onBuildCarouselItem(context, i));
    }
    return widgets;
  }
}
