import 'dart:async';

import 'package:flutter/material.dart';

import '../../constant/value/color.dart';
import '../../constant/value/size.dart';
import 'ripple_wrapper.dart';

class CountdownButtonView extends StatefulWidget {
  final int countdownSec;
  final VoidCallback? onClose;

  const CountdownButtonView(
      {super.key, required this.countdownSec, this.onClose});

  @override
  State<CountdownButtonView> createState() => _CountdownButtonViewState();
}

class _CountdownButtonViewState extends State<CountdownButtonView> {
  late int _countdownRemainSec;
  late Timer _timer;

  @override
  void initState() {
    super.initState();
    _countdownRemainSec = widget.countdownSec;
    _timer = Timer.periodic(
      const Duration(seconds: 1),
      (timer) {
        setState(
          () {
            if (_countdownRemainSec > 0) {
              _countdownRemainSec--;
            } else {
              _timer.cancel();
            }
          },
        );
      },
    );
  }

  @override
  void dispose() {
    _timer.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
      builder: (BuildContext context, BoxConstraints constraints) {
        return Card(
          color: CustomColors.transparent,
          elevation: 0,
          child: RippleWrapper(
            onTap: () {
              if (_countdownRemainSec == 0) {
                widget.onClose?.call();
              }
            },
            borderRadius: BorderRadius.circular(constraints.maxHeight / 2),
            splashColor: CustomColors.gray.withOpacity(0.4),
            decoration: BoxDecoration(
              shape: BoxShape.circle,
              color: CustomColors.midnightBlack.withOpacity(0.4),
              boxShadow: null,
            ),
            child: Center(
              child: _countdownRemainSec > 0
                  ? Text(
                      "$_countdownRemainSec",
                      style: const TextStyle(
                        fontSize: FontSize.extraLarge,
                        color: CustomColors.white,
                      ),
                    )
                  : Icon(
                      Icons.close_rounded,
                      color: CustomColors.white,
                      size: constraints.maxHeight * 0.5,
                    ),
            ),
          ),
        );
      },
    );
  }
}
