import 'dart:async';

import 'package:flutter/material.dart';

class CountingButtonView extends StatefulWidget {
  final int count;
  final VoidCallback onClose;

  const CountingButtonView(this.count, this.onClose, {super.key});

  @override
  State<CountingButtonView> createState() => _CountingButtonViewState();
}

class _CountingButtonViewState extends State<CountingButtonView> {
  late int _count;

  @override
  void initState() {
    super.initState();
    _count = widget.count;
    Timer.periodic(const Duration(seconds: 1), (timer) {
      if (_count > 0) {
        setState(() {
          _count -= 1;
        });
      } else {
        timer.cancel();
      }
    });
  }

  void _onPressed() {
    if (_count == 0) {
      widget.onClose();
    }
  }

  @override
  Widget build(BuildContext context) {
    return Padding(
        padding: const EdgeInsets.fromLTRB(0.0, 16.0, 0.0, 0.0),
        child: SizedBox(
            width: 48.0,
            height: 48.0,
            child: FloatingActionButton(
              backgroundColor: Colors.black87,
              foregroundColor: Colors.grey,
              onPressed: _onPressed,
              child: _count <= 0
                  ? const Icon(Icons.close, size: 32.0)
                  : Text(
                      "$_count",
                      style: const TextStyle(fontSize: 24),
                    ),
            )));
  }
}
