import 'dart:async';

import 'package:flutter/material.dart';

import '../delegate/pagination.dart';

class PaginationView extends StatefulWidget {
  final int length;
  final PaginationDelegate delegate;

  const PaginationView(
      {super.key, required this.length, required this.delegate});

  @override
  State<PaginationView> createState() => _PaginationViewState();
}

class _PaginationViewState extends State<PaginationView> {
  late final StreamController<int> _streamController;

  @override
  void initState() {
    super.initState();
    _streamController = StreamController<int>();
    widget.delegate.onReceiveStreamController(_streamController);
  }

  @override
  void dispose() {
    if (!_streamController.isClosed) {
      _streamController.close();
    }
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<int>(
      stream: _streamController.stream,
      builder: (BuildContext context, AsyncSnapshot<int> snapshot) {
        int activeIndex = snapshot.hasData ? snapshot.data! : 0;
        return Padding(
          padding: const EdgeInsets.only(top: 2),
          child: Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: List.generate(widget.length, (index) {
              return Container(
                width: 6.0,
                height: 6.0,
                margin:
                    const EdgeInsets.symmetric(vertical: 6.0, horizontal: 2.0),
                decoration: BoxDecoration(
                    shape: BoxShape.circle,
                    color: (Theme.of(context).brightness == Brightness.dark
                            ? Colors.white
                            : Colors.grey)
                        .withOpacity(activeIndex == index ? 0.9 : 0.4)),
              );
            }).toList(),
          ),
        );
      },
    );
  }
}
