import 'dart:async';

import 'package:flutter/material.dart';
import 'package:video_player/video_player.dart';

class NativePlayerStatePanelView extends StatefulWidget {
  final VideoPlayerController controller;
  final StreamController<bool> bufferingStateStreamController;
  final StreamController<bool> playingStateStreamController;
  final StreamController<bool> muteStateStreamController;
  final StreamController<bool> finishStateStreamController;
  final StreamController<bool> loopingStateStreamController;
  final StreamController<double> playbackSpeedStateStreamController;
  final StreamController<Duration> positionStateStreamController;
  final StreamController<List<DurationRange>> bufferedStateStreamController;

  const NativePlayerStatePanelView({
    super.key,
    required this.controller,
    required this.bufferingStateStreamController,
    required this.playingStateStreamController,
    required this.muteStateStreamController,
    required this.finishStateStreamController,
    required this.loopingStateStreamController,
    required this.playbackSpeedStateStreamController,
    required this.positionStateStreamController,
    required this.bufferedStateStreamController,
  });

  @override
  State<NativePlayerStatePanelView> createState() =>
      _NativePlayerStatePanelViewState();
}

class _NativePlayerStatePanelViewState
    extends State<NativePlayerStatePanelView> {
  late final VideoPlayerController _controller;
  late bool _isBuffering;
  late bool _isPlaying;
  late bool _isMute;
  late bool _isFinish;
  late bool _isLooping;
  late double _playbackSpeed;
  late Duration _position;
  late List<DurationRange> _buffered;

  @override
  void initState() {
    super.initState();
    _controller = widget.controller;
    _controller.addListener(handlePlayerListener);
    setState(() {
      _isBuffering = _controller.value.isBuffering;
      _isPlaying = _controller.value.isPlaying;
      _isMute = (_controller.value.volume == 0);
      _isFinish = (_controller.value.position == _controller.value.duration);
      _isLooping = _controller.value.isLooping;
      _playbackSpeed = _controller.value.playbackSpeed;
      _position = _controller.value.position;
      _buffered = _controller.value.buffered;
    });
    if (!widget.bufferingStateStreamController.isClosed) {
      widget.bufferingStateStreamController.add(_isBuffering);
    }
    if (!widget.playingStateStreamController.isClosed) {
      widget.playingStateStreamController.add(_isPlaying);
    }
    if (!widget.muteStateStreamController.isClosed) {
      widget.muteStateStreamController.add(_isMute);
    }
    if (!widget.finishStateStreamController.isClosed) {
      widget.finishStateStreamController.add(_isFinish);
    }
    if (!widget.loopingStateStreamController.isClosed) {
      widget.loopingStateStreamController.add(_isLooping);
    }
    if (!widget.playbackSpeedStateStreamController.isClosed) {
      widget.playbackSpeedStateStreamController.add(_playbackSpeed);
    }
    if (!widget.positionStateStreamController.isClosed) {
      widget.positionStateStreamController.add(_position);
    }
    if (!widget.bufferedStateStreamController.isClosed) {
      widget.bufferedStateStreamController.add(_buffered);
    }
  }

  @override
  void dispose() {
    _controller.removeListener(handlePlayerListener);
    super.dispose();
  }

  void handlePlayerListener() {
    if (_controller.value.isBuffering != _isBuffering) {
      setState(() {
        _isBuffering = _controller.value.isBuffering;
      });
      if (!widget.bufferingStateStreamController.isClosed) {
        widget.bufferingStateStreamController
            .add(_controller.value.isBuffering);
      }
    }
    if (_controller.value.isPlaying != _isPlaying) {
      setState(() {
        _isPlaying = _controller.value.isPlaying;
      });
      if (!widget.playingStateStreamController.isClosed) {
        widget.playingStateStreamController.add(_controller.value.isPlaying);
      }
    }
    if ((_controller.value.volume == 0) != _isMute) {
      setState(() {
        _isMute = (_controller.value.volume == 0);
      });
      if (!widget.muteStateStreamController.isClosed) {
        widget.muteStateStreamController.add(_controller.value.volume == 0);
      }
    }
    if ((_controller.value.position == _controller.value.duration) !=
        _isFinish) {
      setState(() {
        _isFinish = (_controller.value.position == _controller.value.duration);
      });
      if (!widget.finishStateStreamController.isClosed) {
        widget.finishStateStreamController
            .add(_controller.value.position == _controller.value.duration);
      }
    }
    if (_controller.value.isLooping != _isLooping) {
      setState(() {
        _isLooping = _controller.value.isLooping;
      });
      if (!widget.loopingStateStreamController.isClosed) {
        widget.loopingStateStreamController.add(_controller.value.isLooping);
      }
    }
    if (_controller.value.playbackSpeed != _playbackSpeed) {
      setState(() {
        _playbackSpeed = _controller.value.playbackSpeed;
      });
      if (!widget.playbackSpeedStateStreamController.isClosed) {
        widget.playbackSpeedStateStreamController
            .add(_controller.value.playbackSpeed);
      }
    }
    if (_controller.value.position != _position) {
      setState(() {
        _position = _controller.value.position;
      });
      if (!widget.positionStateStreamController.isClosed) {
        widget.positionStateStreamController.add(_controller.value.position);
      }
    }
    if (_controller.value.buffered != _buffered) {
      setState(() {
        _buffered = _controller.value.buffered;
      });
      if (!widget.bufferedStateStreamController.isClosed) {
        widget.bufferedStateStreamController.add(_controller.value.buffered);
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    return const SizedBox.shrink();
  }
}
