import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:video_player/video_player.dart';

import '../../../../constant/value/animation.dart';
import '../../../../constant/value/color.dart';

class NativePlayerTexturePanelView extends StatefulWidget {
  final VideoPlayerController controller;
  final Color? backgroundColor;

  const NativePlayerTexturePanelView({
    super.key,
    required this.controller,
    this.backgroundColor,
  });

  @override
  State<NativePlayerTexturePanelView> createState() =>
      _NativePlayerTexturePanelViewState();
}

class _NativePlayerTexturePanelViewState
    extends State<NativePlayerTexturePanelView> {
  late final VideoPlayerController _controller;
  late bool _isPlaying;
  late bool shouldShowFrame;
  late double aspectRatio;

  @override
  void initState() {
    super.initState();
    _controller = widget.controller;
    _controller.addListener(handlePlayerListener);
    setState(() {
      aspectRatio = _controller.value.aspectRatio;
      _isPlaying = _controller.value.isPlaying;
      shouldShowFrame = true;
    });
  }

  @override
  void dispose() {
    _controller.removeListener(handlePlayerListener);
    super.dispose();
  }

  void handlePlayerListener() {
    if (_controller.value.isPlaying != _isPlaying) {
      setState(() {
        shouldShowFrame = false;
      });
      Future.delayed(AnimationDuration.short, () {
        setState(() {
          shouldShowFrame = true;
        });
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      color: widget.backgroundColor ?? CustomColors.black,
      child: Center(
        child: AspectRatio(
          aspectRatio: aspectRatio,
          child: shouldShowFrame
              ? VideoPlayer(_controller)
              : const SizedBox.shrink(),
        ),
      ),
    );
  }
}
