import 'dart:async';

import 'package:flutter/material.dart';

import '../../../../constant/enum/transition.dart';
import '../../../../constant/value/animation.dart';
import '../../../../constant/value/color.dart';
import '../../transition_wrapper.dart';

class PlayerLoadingPanelView extends StatelessWidget {
  final StreamController<bool> bufferingStateStreamController;

  const PlayerLoadingPanelView(
      {super.key, required this.bufferingStateStreamController});

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<bool>(
        initialData: true,
        stream: bufferingStateStreamController.stream,
        builder: (BuildContext bufferedStateContext,
            AsyncSnapshot<bool> bufferedStateSnapshot) {
          return _getPlayerLoadingPanelWidget(bufferedStateSnapshot.data!);
        });
  }

  Widget _getPlayerLoadingPanelWidget(bool bufferingState) {
    return TransitionWrapper(
      transitionKey:
          bufferingState ? TransitionKey.fadeIn : TransitionKey.fadeOut,
      duration: AnimationDuration.short,
      child: IgnorePointer(
        ignoring: !bufferingState,
        child: Container(
          color: CustomColors.transparent,
          child: const Center(
            child: SizedBox(
              height: 35,
              width: 35,
              child: CircularProgressIndicator(
                strokeWidth: 3,
                valueColor:
                    AlwaysStoppedAnimation<Color>(CustomColors.lightGray),
              ),
            ),
          ),
        ),
      ),
    );
  }
}
