import 'dart:async';

import 'package:flutter/material.dart';

import '../../../../constant/enum/transition.dart';
import '../../../../constant/value/animation.dart';
import '../../../../constant/value/color.dart';
import '../../base64_file_image_provider.dart';
import '../../ripple_wrapper.dart';
import '../../transition_wrapper.dart';

class PlayerPauseImagePanelView extends StatelessWidget {
  final StreamController<bool> playingStateStreamController;
  final StreamController<bool> finishStateStreamController;
  final String pauseImagePath;
  final VoidCallback? onClose;
  final VoidCallback? onTap;

  const PlayerPauseImagePanelView({
    super.key,
    required this.playingStateStreamController,
    required this.finishStateStreamController,
    required this.pauseImagePath,
    this.onClose,
    this.onTap,
  });

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<bool>(
        initialData: true,
        stream: playingStateStreamController.stream,
        builder: (BuildContext playingStateContext,
            AsyncSnapshot<bool> playingStateSnapshot) {
          return StreamBuilder<bool>(
              initialData: false,
              stream: finishStateStreamController.stream,
              builder: (BuildContext finishStateContext,
                  AsyncSnapshot<bool> finishStateSnapshot) {
                return _getPlayerPauseImagePanelWidget(
                    playingStateSnapshot.data!, finishStateSnapshot.data!);
              });
        });
  }

  Widget _getPlayerPauseImagePanelWidget(bool playingState, bool finishState) {
    return TransitionWrapper(
      transitionKey: !playingState && !finishState
          ? TransitionKey.fadeIn
          : TransitionKey.fadeOut,
      duration: AnimationDuration.short,
      child: IgnorePointer(
        ignoring: playingState || finishState,
        child: Container(
          color: CustomColors.midnightBlack,
          child: Stack(
            children: [
              LayoutBuilder(
                  builder: (BuildContext context, BoxConstraints constraints) {
                return GestureDetector(
                  onTap: onTap,
                  child: Image(
                    width: constraints.maxWidth,
                    height: constraints.maxHeight,
                    image: Base64FileImageProvider(
                      pauseImagePath,
                    ),
                    fit: BoxFit.contain,
                  ),
                );
              }),
              Positioned(
                width: 50,
                height: 50,
                top: 5,
                right: 5,
                child: LayoutBuilder(
                  builder: (BuildContext context, BoxConstraints constraints) {
                    return Card(
                      color: CustomColors.transparent,
                      elevation: 0,
                      child: RippleWrapper(
                        onTap: onClose,
                        borderRadius:
                            BorderRadius.circular(constraints.maxHeight / 2),
                        splashColor: CustomColors.gray.withOpacity(0.4),
                        decoration: BoxDecoration(
                          shape: BoxShape.circle,
                          color: CustomColors.midnightBlack.withOpacity(0.4),
                          boxShadow: null,
                        ),
                        child: Center(
                          child: Icon(
                            Icons.close_rounded,
                            color: CustomColors.white,
                            size: constraints.maxHeight * 0.5,
                          ),
                        ),
                      ),
                    );
                  },
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }
}
