import 'package:flutter/material.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';

class RefreshWrapper extends StatefulWidget {
  final Widget child;
  final bool enableRefresh, enableLoading;
  final VoidFutureCallBack? onRefresh, onLoading;

  const RefreshWrapper(
      {required this.child,
      required this.enableRefresh,
      required this.enableLoading,
      this.onRefresh,
      this.onLoading,
      super.key});

  @override
  State<RefreshWrapper> createState() => _RefreshWrapperState();
}

class _RefreshWrapperState extends State<RefreshWrapper> {
  final RefreshController _refreshController = RefreshController();

  @override
  Widget build(BuildContext context) {
    return ScrollConfiguration(
      behavior: ScrollConfiguration.of(context).copyWith(scrollbars: false),
      child: SmartRefresher(
          physics: const BouncingScrollPhysics(),
          controller: _refreshController,
          header: const ClassicHeader(),
          footer: const ClassicFooter(),
          enablePullDown: widget.enableRefresh,
          enablePullUp: widget.enableLoading,
          onRefresh: _onRefresh,
          onLoading: _onLoading,
          child: widget.child),
    );
  }

  void _onRefresh() async {
    if (widget.onRefresh != null) {
      await widget.onRefresh!().whenComplete(() {
        setState(() {
          _refreshController.refreshCompleted();
        });
      });
    } else {
      setState(() {
        _refreshController.refreshCompleted();
      });
    }
  }

  void _onLoading() async {
    if (widget.onLoading != null) {
      await widget.onLoading!().whenComplete(() {
        setState(() {
          _refreshController.loadComplete();
        });
      });
    } else {
      setState(() {
        _refreshController.loadComplete();
      });
    }
  }
}
