import 'dart:async';

import 'package:flutter/material.dart';

import '../../constant/enum/transition.dart';

class TransitionWrapper extends StatefulWidget {
  final TransitionKey transitionKey;
  final Duration duration;
  final Widget child;

  const TransitionWrapper(
      {super.key,
      required this.transitionKey,
      required this.duration,
      required this.child});

  @override
  State<StatefulWidget> createState() => _TransitionWrapperState();
}

class _TransitionWrapperState extends State<TransitionWrapper>
    with TickerProviderStateMixin {
  late Future<void> future;
  final GlobalKey<_TransitionWrapperState> key = GlobalKey();

  @override
  void initState() {
    super.initState();
    setState(() {
      future = getDurationFuture();
    });
  }

  Future<void> getDurationFuture() {
    Completer completer = Completer();
    Future.delayed(widget.duration).whenComplete(() => completer.complete());
    return completer.future;
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<void>(
        future: future,
        builder: (BuildContext context, AsyncSnapshot<void> snapshot) {
          return getTransitionWidget(
              snapshot.connectionState == ConnectionState.done);
        });
  }

  Widget getTransitionWidget(bool animated) {
    switch (widget.transitionKey) {
      case TransitionKey.none:
        return widget.child;
      case TransitionKey.slideInFromLeft:
        return AnimatedSlide(
          offset: animated ? Offset.zero : const Offset(-1.0, 0.0),
          duration: widget.duration,
          curve: Curves.easeInOut,
          child: widget.child,
        );
      case TransitionKey.slideInFromRight:
        return AnimatedSlide(
          offset: animated ? Offset.zero : const Offset(1.0, 0.0),
          duration: widget.duration,
          curve: Curves.easeInOut,
          child: widget.child,
        );

      case TransitionKey.slideInFromTop:
        return AnimatedSlide(
          offset: animated ? Offset.zero : const Offset(0.0, -1.0),
          duration: widget.duration,
          curve: Curves.easeInOut,
          child: widget.child,
        );
      case TransitionKey.slideInFromBottom:
        return AnimatedSlide(
          offset: animated ? Offset.zero : const Offset(0.0, 1.0),
          duration: widget.duration,
          curve: Curves.easeInOut,
          child: widget.child,
        );
      case TransitionKey.slideOutToLeft:
        return AnimatedSlide(
          offset: animated ? const Offset(-1.0, 0.0) : Offset.zero,
          duration: widget.duration,
          curve: Curves.easeInOut,
          child: widget.child,
        );
      case TransitionKey.slideOutToRight:
        return AnimatedSlide(
          offset: animated ? const Offset(1.0, 0.0) : Offset.zero,
          duration: widget.duration,
          curve: Curves.easeInOut,
          child: widget.child,
        );
      case TransitionKey.slideOutToTop:
        return AnimatedSlide(
          offset: animated ? const Offset(0.0, -1.0) : Offset.zero,
          duration: widget.duration,
          curve: Curves.easeInOut,
          child: widget.child,
        );
      case TransitionKey.slideOutToBottom:
        return AnimatedSlide(
          offset: animated ? const Offset(0.0, 1.0) : Offset.zero,
          duration: widget.duration,
          curve: Curves.easeInOut,
          child: widget.child,
        );
      case TransitionKey.fadeIn:
        return AnimatedOpacity(
          opacity: animated ? 1.0 : 0.0,
          duration: widget.duration,
          child: widget.child,
        );
      case TransitionKey.fadeOut:
        return AnimatedOpacity(
          opacity: animated ? 0.0 : 1.0,
          duration: widget.duration,
          child: widget.child,
        );
    }
  }
}
