import 'package:flutter/material.dart';
import 'package:flutter_localization/flutter_localization.dart';
import 'package:redinfo_integrated_ott_platform_flutter_sdk/delegate/communication_output_delegate.dart';

import '../../constant/value/color.dart';
import '../../constant/value/instance_container.dart';
import '../../delegate/data_service.dart';
import 'component/delegate/player_switch.dart';
import 'constant/value/assets.dart';
import 'constant/value/localization.dart';
import 'delegate/load_balancer.dart';
import 'delegate/route.dart';
import 'manager/data_service.dart';
import 'manager/iframe_communication.dart';
import 'manager/load_balancer/load_balancer.dart' as load_balancer;
import 'manager/route.dart';
import 'util/assets.dart';
import 'util/instance_register.dart';
import 'util/localization.dart';

void main() async {
  // initial ui
  WidgetsFlutterBinding.ensureInitialized();
  // initial localization
  final FlutterLocalization localization = FlutterLocalization.instance;
  Map<String, dynamic> rootCnMap = await Localization.mapFromJsonFile(
      assets: Assets.getAssetsPath(AssetsPath.i18ZhCn));
  localization.init(mapLocales: [
    MapLocale('zh', rootCnMap, countryCode: "CN"),
  ], initLanguageCode: 'zh', initCountryCode: "CN");
  //initial error listener
  FlutterError.onError = (details) {
    FlutterError.presentError(details);
  };
  //generate manager
  load_balancer.LoadBalancerManager loadBalancerManager =
  load_balancer.LoadBalancerManager();
  DataServiceManager dataServiceManager = DataServiceManager();
  RouteManager routeManager = RouteManager();
  IframeCommunicationManager communicationManager =
  IframeCommunicationManager();
  // register manager as delegate
  InstanceRegister.registerSingleton<DataServiceDelegate>(
          (container) => dataServiceManager,
      name: InstanceContainer.dataServiceDelegate);
  InstanceRegister.registerSingleton<RouteDelegate>((container) => routeManager,
      name: InstanceContainer.routeDelegate);
  InstanceRegister.registerSingleton<LoadBalancerDelegate>(
          (container) => loadBalancerManager,
      name: InstanceContainer.loadBalancerDelegate);
  InstanceRegister.registerSingleton<CommunicationOutputDelegate>(
          (container) => communicationManager,
      name: InstanceContainer.communicationOutputDelegate);
  InstanceRegister.registerSingleton<Map<String, PlayerSwitchDelegate>>(
          (container) => {},
      name: InstanceContainer.playerSwitchDelegateMap);
  // initial
  await dataServiceManager.initial();
  // run
  runApp(
    _Application(
      title: rootCnMap[LocalizationStrings.appName],
      localization: localization,
      rootNavigatorKey: routeManager.rootNavigatorKey,
      rootNavigator: routeManager.getRootNavigator(withPopScope: true),
    ),
  );
}

class _Application extends StatelessWidget {
  final String title;
  final FlutterLocalization localization;
  final GlobalKey<NavigatorState> rootNavigatorKey;
  final Widget rootNavigator;

  const _Application({
    required this.title,
    required this.localization,
    required this.rootNavigatorKey,
    required this.rootNavigator,
  });

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      color: CustomColors.black,
      navigatorKey: rootNavigatorKey,
      title: title,
      localizationsDelegates: localization.localizationsDelegates,
      supportedLocales: localization.supportedLocales,
      theme: ThemeData(
        useMaterial3: true,
        splashColor: CustomColors.transparent,
        highlightColor: CustomColors.transparent,
        hoverColor: CustomColors.transparent,
        scaffoldBackgroundColor: CustomColors.black,
      ),
      home: rootNavigator,
    );
  }
}