import 'package:flutter/material.dart';

import '../../component/delegate/carousel.dart';
import '../../component/view/base64_file_image_provider.dart';
import '../../component/view/carousel.dart';
import '../../structure/entity/advertisement_image.dart';
import '../../util/intent.dart';
import 'contract.dart';

class AdvertisementCarouselView extends StatelessWidget
    implements CarouselDelegate {
  final AdvertisementCarouselPresenterContract presenter;
  final List<AdvertisementImage> list;

  const AdvertisementCarouselView(
      {super.key, required this.presenter, required this.list});

  @override
  Widget build(BuildContext context) {
    return CarouselView(delegate: this);
  }

  @override
  int getClickCarouselItemSize() {
    return list.length;
  }

  @override
  Widget onBuildCarouselItem(BuildContext context, int index) {
    AdvertisementImage advertisementImage = list[index % list.length];
    return GestureDetector(
      onTap: () => _onTapAdvertisementImage(index),
      child: Image(
        image: Base64FileImageProvider(
          advertisementImage.imagePath,
        ),
        fit: BoxFit.contain,
      ),
    );
  }

  @override
  void onSlide(int index) {
    presenter.onCarouselSlide(index);
  }

  void _onTapAdvertisementImage(int index) {
    if (index <= list.length - 1) {
      IntentDistributor.fireBrowserIntent(list[index].link);
    }
  }
}
