import 'package:flutter/material.dart';

import '../../component/view/base64_file_image_provider.dart';
import '../../component/view/transition_wrapper.dart';
import '../../constant/enum/transition.dart';
import '../../constant/value/animation.dart';
import '../../structure/entity/advertisement_image.dart';
import '../../util/intent.dart';
import 'contract.dart';

class AdvertisementImageView extends StatelessWidget {
  final AdvertisementImagePresenterContract presenter;
  final AdvertisementImage advertisementImage;

  const AdvertisementImageView(
      {super.key, required this.presenter, required this.advertisementImage});

  @override
  Widget build(BuildContext context) {
    return TransitionWrapper(
      transitionKey: TransitionKey.fadeIn,
      duration: AnimationDuration.short,
      child: onBuildItem(),
    );
  }

  Widget onBuildItem() {
    return GestureDetector(
      onTap: _onTapAdvertisementImage,
      child: Image(
        image: Base64FileImageProvider(
          advertisementImage.imagePath,
        ),
        fit: BoxFit.contain,
      ),
    );
  }

  void _onTapAdvertisementImage() {
    IntentDistributor.fireBrowserIntent(advertisementImage.link);
  }
}
