import 'package:flutter/material.dart';

import '../../component/view/ripple_wrapper.dart';
import '../../constant/value/color.dart';
import '../../constant/value/localization.dart';
import '../../constant/value/size.dart';
import '../../structure/entity/video_group.dart';
import '../../util/localization.dart';
import 'contract.dart';

class ClassificationMenuBarView extends StatelessWidget {
  final ClassificationMenuBarPresenterContract presenter;
  final List<VideoGroup> videoGroupList;

  const ClassificationMenuBarView(
      {super.key, required this.presenter, required this.videoGroupList});

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.symmetric(vertical: 4, horizontal: 2),
      child: SingleChildScrollView(
        scrollDirection: Axis.horizontal,
        child: Row(
          children: List.generate(
            videoGroupList.length,
            (index) => _getMenuItemWidget(context, videoGroupList[index]),
          ),
        ),
      ),
    );
  }

  Widget _getMenuItemWidget(BuildContext context, VideoGroup videoGroup) {
    return Padding(
      padding: const EdgeInsets.symmetric(horizontal: 2),
      child: Card(
        child: RippleWrapper(
          borderRadius: BorderRadius.circular(5),
          splashColor: CustomColors.emeraldGreen,
          onTap: () => presenter.navigateGroupSearchPage(videoGroup.id),
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(5),
            border: Border.all(color: CustomColors.lightMint),
            gradient: const LinearGradient(
              begin: Alignment.centerLeft,
              end: Alignment.centerRight,
              stops: [0, 1],
              colors: [
                CustomColors.deepGreen,
                CustomColors.green,
              ],
            ),
          ),
          child: Padding(
            padding: const EdgeInsets.symmetric(vertical: 6, horizontal: 24),
            child: Column(
              children: [
                Text(
                  videoGroup.title,
                  style: const TextStyle(
                    fontWeight: FontWeight.bold,
                    fontSize: FontSize.large,
                    color: CustomColors.white,
                  ),
                ),
                Text(
                  "${videoGroup.collectionCount}\u0020${Localization.getString(context, LocalizationStrings.collectionCountSuffix)}",
                  style: const TextStyle(
                    fontSize: FontSize.medium,
                    color: CustomColors.white,
                  ),
                ),
              ],
            ),
          ),
        ),
      ),
    );
  }
}
