import 'dart:async';
import 'package:flutter/material.dart';
import '../../bloc/advertisement_image/bloc.dart';
import '../../bloc/advertisement_image/delegate.dart';
import '../../bloc/advertisement_image/event.dart';
import '../../bloc/advertisement_image/state.dart';
import '../../bloc/classification_detail/bloc.dart';
import '../../bloc/classification_detail/delegate.dart';
import '../../bloc/classification_detail/event.dart';
import '../../bloc/classification_detail/state.dart';
import '../../constant/enum/route.dart';
import '../../constant/value/image.dart';
import '../../constant/value/instance_container.dart';
import '../../constant/value/length.dart';
import '../../constant/value/module_param.dart';
import '../../constant/value/route.dart';
import '../../delegate/data_service.dart';
import '../../delegate/route.dart';
import '../../structure/entity/advertisement_image.dart';
import '../../structure/entity/classification_detail.dart';
import '../../structure/entity/video.dart';
import '../../structure/entity/video_group.dart';
import '../../util/instance_register.dart';
import '../presenter.dart';
import '../video_collection/contract.dart';
import 'contract.dart';
import 'view.dart';

class ClassificationPagePresenter extends BasePresenter
    implements
        ClassificationPagePresenterContract,
        AdvertisementImageBlocDelegate,
        ClassificationDetailBlocDelegate,
        VideoCollectionModuleDelegate {
  static const int offsetLength = 4;
  AdvertisementImageBloc? advertisementImageBloc;
  ClassificationDetailBloc? classificationDetailBloc;

  @override
  Widget createView(Map<String, dynamic> params) {
    advertisementImageBloc ??= AdvertisementImageBloc(
        delegate: this, initialState: AdvertisementImageBlocState.init());
    classificationDetailBloc ??= ClassificationDetailBloc(
        delegate: this, initialState: ClassificationDetailBlocState.init());
    return ClassificationPageView(
      presenter: this,
      advertisementImageBloc: advertisementImageBloc,
      classificationDetailBloc: classificationDetailBloc,
    );
  }

  @override
  void initialViewData() {
    advertisementImageBloc?.add(AdvertisementImageListMapFetchEvent());
    classificationDetailBloc?.add(ClassificationDetailFetchEvent());
  }

  @override
  Widget getAdvertisementCarouselWidget(
      List<AdvertisementImage> advertisementImageList) {
    RouteDelegate routeDelegate = InstanceRegister.resolve<RouteDelegate>(
        InstanceContainer.routeDelegate);
    return routeDelegate.getViewModule(ModuleViewKey.advertisementCarousel, {
      ModuleParamKey.advertisementImageList: advertisementImageList,
    });
  }

  @override
  Widget getAdvertisementImageWidget(AdvertisementImage advertisementImage) {
    RouteDelegate routeDelegate = InstanceRegister.resolve<RouteDelegate>(
        InstanceContainer.routeDelegate);
    return routeDelegate.getViewModule(ModuleViewKey.advertisementImage, {
      ModuleParamKey.advertisementImage: advertisementImage,
    });
  }

  @override
  Widget getClassificationMenuBarWidget(List<VideoGroup> videoGroupList) {
    RouteDelegate routeDelegate = InstanceRegister.resolve<RouteDelegate>(
        InstanceContainer.routeDelegate);
    return routeDelegate.getViewModule(ModuleViewKey.classificationMenuBar, {
      ModuleParamKey.videoGroupList: videoGroupList,
    });
  }

  @override
  Widget getVideoCollectionWidget(List<Video> videoList) {
    RouteDelegate routeDelegate = InstanceRegister.resolve<RouteDelegate>(
        InstanceContainer.routeDelegate);
    return routeDelegate.getViewModule(ModuleViewKey.videoCollection, {
      ModuleParamKey.moduleDelegate: this,
      ModuleParamKey.crossAxisCount: 3,
      ModuleParamKey.enableScroll: false,
      ModuleParamKey.videoList: videoList,
    });
  }

  @override
  Future<Map<String, List<AdvertisementImage>>>
      fetchAdvertisementImageListMap() {
    DataServiceDelegate dataServiceDelegate =
        InstanceRegister.resolve<DataServiceDelegate>(
            InstanceContainer.dataServiceDelegate);
    return dataServiceDelegate.getAdvertisementImageListMap(
        [ImageRatio.ratio_2_1, ImageRatio.ratio_3_1]).then((map) {
      List<AdvertisementImage> ration_2_1List = map[ImageRatio.ratio_2_1] ?? [];
      List<AdvertisementImage> ration_3_1List = map[ImageRatio.ratio_3_1] ?? [];
      return {
        ImageRatio.ratio_2_1: ration_2_1List,
        ImageRatio.ratio_3_1: ration_3_1List..shuffle(),
      };
    });
  }

  @override
  Future<ClassificationDetail> fetchClassificationDetail() {
    DataServiceDelegate dataServiceDelegate =
        InstanceRegister.resolve<DataServiceDelegate>(
            InstanceContainer.dataServiceDelegate);
    return dataServiceDelegate.getClassificationDetail(
        ListLength.medium, ImageRatio.ratio_3_4);
  }

  @override
  void onFireTapVideoEvent(int id) {
    RouteDelegate routeDelegate = InstanceRegister.resolve<RouteDelegate>(
        InstanceContainer.routeDelegate);
    routeDelegate.push(RoutePath.media, {
      ModuleParamKey.videoId: id,
    });
  }
}
