import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:movie_flutter/structure/entity/classification_video.dart';
import 'package:movie_flutter/structure/entity/video.dart';

import '../../bloc/advertisement_image/bloc.dart';
import '../../bloc/advertisement_image/state.dart';
import '../../bloc/classification_detail/bloc.dart';
import '../../bloc/classification_detail/state.dart';
import '../../component/view/refresh_wrapper.dart';
import '../../constant/value/color.dart';
import '../../constant/value/image.dart';
import '../../constant/value/length.dart';
import '../../structure/entity/advertisement_image.dart';
import 'contract.dart';

class ClassificationPageView extends StatelessWidget {
  static const blockVideoLength = ListLength.small;
  final ClassificationPagePresenterContract presenter;
  final AdvertisementImageBloc? advertisementImageBloc;
  final ClassificationDetailBloc? classificationDetailBloc;

  const ClassificationPageView({
    super.key,
    required this.presenter,
    required this.advertisementImageBloc,
    required this.classificationDetailBloc,
  });

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: CustomColors.black,
      body: RefreshWrapper(
        enableRefresh: false,
        enableLoading: false,
        child: SingleChildScrollView(
          child: Column(
            children: [
              _getClassificationMenuBarWidget(),
              _getClassificationVideoListWidget(),
            ],
          ),
        ),
      ),
    );
  }

  Widget _getClassificationMenuBarWidget() {
    return BlocBuilder<ClassificationDetailBloc, ClassificationDetailBlocState>(
      bloc: classificationDetailBloc,
      builder: (context, state) {
        if (state.classificationDetail == null) {
          return const SizedBox.shrink();
        }
        return presenter.getClassificationMenuBarWidget(
            state.classificationDetail!.videoGroupList);
      },
    );
  }

  Widget _getClassificationVideoListWidget() {
    return BlocBuilder<ClassificationDetailBloc, ClassificationDetailBlocState>(
      bloc: classificationDetailBloc,
      builder: (context, state) {
        if (state.classificationDetail == null) {
          return const SizedBox.shrink();
        }
        return ListView.builder(
          physics: const NeverScrollableScrollPhysics(),
          shrinkWrap: true,
          itemCount:
              (state.classificationDetail!.classificationVideoList.length /
                              blockVideoLength)
                          .ceil() *
                      2 -
                  1,
          itemBuilder: (BuildContext context, int index) {
            int itemIndex = (index / 2).floor();
            if (index % 2 == 0) {
              List<ClassificationVideo> classificationVideoList =
                  state.classificationDetail!.classificationVideoList.sublist(
                      itemIndex * blockVideoLength,
                      (itemIndex + 1) * blockVideoLength);
              return presenter.getVideoCollectionWidget(List.generate(
                  classificationVideoList.length, (classificationVideoIndex) {
                ClassificationVideo classificationVideo =
                    classificationVideoList[classificationVideoIndex];
                return Video(
                    id: classificationVideo.id,
                    title: classificationVideo.title,
                    thumbnailPath: classificationVideo.thumbnailPath);
              }));
            }
            return _getAdvertisementImageWidget(itemIndex);
          },
        );
      },
    );
  }

  Widget _getAdvertisementImageWidget(int index) {
    return BlocBuilder<AdvertisementImageBloc, AdvertisementImageBlocState>(
      bloc: advertisementImageBloc,
      builder: (context, state) {
        if (!state.advertisementImageListMap
            .containsKey(ImageRatio.ratio_3_1)) {
          return const SizedBox.shrink();
        }
        List<AdvertisementImage> advertisementImageList =
            state.advertisementImageListMap[ImageRatio.ratio_3_1]!.toList();
        AdvertisementImage advertisementImage =
            advertisementImageList[index % advertisementImageList.length];
        return AspectRatio(
          aspectRatio: ImageAspectRatio.ratio_3_1,
          child: presenter.getAdvertisementImageWidget(advertisementImage),
        );
      },
    );
  }
}
