import 'package:flutter/material.dart';

import '../../constant/value/color.dart';
import '../../constant/value/size.dart';
import '../../structure/entity/video_group.dart';
import 'contract.dart';

class GroupSearchGroupTabView extends StatefulWidget {
  final GroupSearchGroupTabPresenterContract presenter;
  final List<VideoGroup> videoGroupLst;
  final int initialGroupId;

  const GroupSearchGroupTabView({
    super.key,
    required this.presenter,
    required this.videoGroupLst,
    required this.initialGroupId,
  });

  @override
  State<GroupSearchGroupTabView> createState() =>
      _GroupSearchGroupTabViewState();
}

class _GroupSearchGroupTabViewState extends State<GroupSearchGroupTabView>
    with TickerProviderStateMixin {
  late TabController _tabController;
  int _currentIndex = 0;

  @override
  void initState() {
    super.initState();
    final index = widget.videoGroupLst
        .indexWhere((videoGroup) => videoGroup.id == widget.initialGroupId);
    if (index >= 0) {
      _currentIndex = index;
    }
    _tabController = TabController(
        initialIndex: _currentIndex,
        length: widget.videoGroupLst.length,
        vsync: this);
    _tabController.addListener(_handleTabSelection);
  }

  @override
  void dispose() {
    super.dispose();
    _tabController.dispose();
  }

  void _handleTabSelection() {
    if (!_tabController.indexIsChanging) {
      widget.presenter
          .onSelectGroup(widget.videoGroupLst[_tabController.index]);
      setState(() {
        _currentIndex = _tabController.index;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return TabBar(
      controller: _tabController,
      dividerColor:Colors.transparent,
      indicatorColor: Colors.transparent,
      padding: const EdgeInsets.all(8),
      labelPadding: const EdgeInsets.symmetric(horizontal: 6),
      splashFactory: NoSplash.splashFactory,
      splashBorderRadius: BorderRadius.circular(12),
      tabs: widget.videoGroupLst
          .asMap()
          .map((index, item) => MapEntry(
                index,
                _getTabBarItemWidget(
                    active: _currentIndex == index, videoGroup: item),
              ))
          .values
          .toList(),
      isScrollable: true,
    );
  }

  Widget _getTabBarItemWidget(
      {required bool active, required VideoGroup videoGroup}) {
    return Padding(
      padding: EdgeInsets.zero,
      child: Container(
        decoration: BoxDecoration(
          color: active ? CustomColors.green : CustomColors.white,
          borderRadius: BorderRadius.circular(16),
        ),
        child: Padding(
          padding: const EdgeInsets.symmetric(vertical: 4, horizontal: 16),
          child: Center(
            child: Text(
              videoGroup.title,
              style: TextStyle(
                color: active ? CustomColors.white : CustomColors.green,
                fontWeight: FontWeight.bold,
                fontSize: FontSize.medium,
              ),
            ),
          ),
        ),
      ),
    );
  }
}
