import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import '../../bloc/video/bloc.dart';
import '../../bloc/video/state.dart';
import '../../bloc/video_group/bloc.dart';
import '../../bloc/video_group/state.dart';
import '../../component/view/refresh_wrapper.dart';
import '../../component/view/transition_wrapper.dart';
import '../../constant/enum/transition.dart';
import '../../constant/value/animation.dart';
import '../../constant/value/color.dart';
import 'contract.dart';

class GroupSearchPageView extends StatelessWidget {
  final GroupSearchPagePresenterContract presenter;
  final VideoGroupBloc? groupBloc;
  final VideoBloc? videoBloc;

  const GroupSearchPageView(
      {super.key,
      required this.presenter,
      required this.groupBloc,
      required this.videoBloc});

  @override
  Widget build(BuildContext context) {
    double tabHeight = MediaQuery.of(context).size.height * 0.07;
    return Scaffold(
      backgroundColor: CustomColors.black,
      body: BlocBuilder<VideoBloc, VideoBlocState>(
        bloc: videoBloc,
        builder: (context, state) {
          return RefreshWrapper(
            enableRefresh: false,
            enableLoading: !state.finish,
            onRefresh: presenter.onFireRefreshEvent,
            onLoading: presenter.onFireLoadingEvent,
            child: CustomScrollView(
              cacheExtent: 500,
              slivers: <Widget>[
                _getGroupSearchSortTabWidget(tabHeight),
                _getGroupSearchGroupTabWidget(),
                _getVideoCollectionWidget(),
              ],
            ),
          );
        },
      ),
    );
  }

  Widget _getGroupSearchSortTabWidget(double height) {
    return SliverToBoxAdapter(
      child: SizedBox(
        height: height,
        child: TransitionWrapper(
          transitionKey: TransitionKey.slideInFromTop,
          duration: AnimationDuration.short,
          child: presenter.getGroupSearchSortTabWidget(),
        ),
      ),
    );
  }

  Widget _getGroupSearchGroupTabWidget() {
    return BlocBuilder<VideoGroupBloc, VideoGroupBlocState>(
      bloc: groupBloc,
      builder: (context, state) {
        if (state.videoGroupList.isEmpty) {
          return const SliverToBoxAdapter(
            child: SizedBox.shrink(),
          );
        }
        return SliverToBoxAdapter(
          child: TransitionWrapper(
            transitionKey: TransitionKey.slideInFromLeft,
            duration: AnimationDuration.short,
            child: presenter.getGroupSearchGroupTabWidget(state.videoGroupList),
          ),
        );
      },
    );
  }

  Widget _getVideoCollectionWidget() {
    return BlocBuilder<VideoBloc, VideoBlocState>(
      bloc: videoBloc,
      builder: (context, state) {
        if (state.videoList.isEmpty) {
          return const SliverToBoxAdapter(
            child: SizedBox.shrink(),
          );
        }
        return SliverPadding(
          padding: const EdgeInsets.all(4.0),
          sliver: presenter.getVideoCollectionWidget(state.videoList),
        );
      },
    );
  }
}
