import 'package:flutter/material.dart';

import '../../constant/value/color.dart';
import '../../constant/value/size.dart';
import '../../structure/entity/video_sort.dart';
import '../../util/localization.dart';
import 'contract.dart';

class GroupSearchSortTabView extends StatefulWidget {
  final GroupSearchSortTabPresenterContract presenter;
  final List<VideoSort> videoSortLst;

  const GroupSearchSortTabView({
    super.key,
    required this.presenter,
    required this.videoSortLst,
  });

  @override
  State<GroupSearchSortTabView> createState() => _GroupSearchSortTabViewState();
}

class _GroupSearchSortTabViewState extends State<GroupSearchSortTabView>
    with TickerProviderStateMixin {
  late TabController _tabController;
  int _currentIndex = 0;

  @override
  void initState() {
    super.initState();
    _tabController =
        TabController(length: widget.videoSortLst.length, vsync: this);
    _tabController.addListener(_handleTabSelection);
  }

  @override
  void dispose() {
    super.dispose();
    _tabController.dispose();
  }

  void _handleTabSelection() {
    if (!_tabController.indexIsChanging) {
      widget.presenter.onSelectSort(widget.videoSortLst[_tabController.index]);
      setState(() {
        _currentIndex = _tabController.index;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return TabBar(
      controller: _tabController,
      dividerColor: Colors.transparent,
      indicatorColor: Colors.transparent,
      labelPadding: EdgeInsets.zero,
      padding: EdgeInsets.zero,
      tabs: widget.videoSortLst
          .asMap()
          .map((index, item) => MapEntry(
                index,
                _getTabBarItemWidget(
                    active: _currentIndex == index, videoSort: item),
              ))
          .values
          .toList(),
      isScrollable: false,
    );
  }

  Widget _getTabBarItemWidget(
      {required bool active, required VideoSort videoSort}) {
    return Container(
      width: double.infinity,
      height: double.infinity,
      color: active ? CustomColors.green : CustomColors.white,
      child: Center(
        child: Text(
          Localization.getString(context, videoSort.title),
          style: TextStyle(
            fontSize: FontSize.medium,
            color: active ? CustomColors.white : CustomColors.green,
          ),
        ),
      ),
    );
  }
}
