import 'package:flutter/material.dart';
import '../../constant/value/color.dart';
import '../../constant/value/localization.dart';
import '../../constant/value/punctuation_marks.dart';
import '../../constant/value/size.dart';
import '../../structure/entity/video_detail.dart';
import '../../util/format_parser.dart';
import '../../util/localization.dart';
import 'contract.dart';

class MediaDetailView extends StatefulWidget {
  final MediaDetailPresenterContract presenter;
  final VideoDetail videoDetail;

  const MediaDetailView({
    super.key,
    required this.presenter,
    required this.videoDetail,
  });

  @override
  State<MediaDetailView> createState() => _MediaDetailViewState();
}

class _MediaDetailViewState extends State<MediaDetailView> {
  bool _isExpanded = false;

  @override
  Widget build(BuildContext context) {
    return ExpansionPanelList(
      expandIconColor: CustomColors.gray,
      expansionCallback: _onExpansionCallback,
      expandedHeaderPadding: const EdgeInsets.all(0),
      children: [
        _getDetailExpansionPanel(),
      ],
    );
  }

  ExpansionPanel _getDetailExpansionPanel() {
    return ExpansionPanel(
      backgroundColor: CustomColors.midnightBlack,
      headerBuilder: (BuildContext context, bool isExpanded) {
        return ListTile(
          title: Text(
            Localization.getString(context, LocalizationStrings.detail),
            style: const TextStyle(
              fontSize: FontSize.medium,
              color: CustomColors.gray,
            ),
          ),
        );
      },
      body: Padding(
        padding: const EdgeInsets.symmetric(horizontal: 16),
        child: Wrap(
          children: [
            Container(
              padding: const EdgeInsets.only(bottom: 4),
              width: double.infinity,
              child: Text(
                '${Localization.getString(context, LocalizationStrings.group)} \u003A ${List.generate(widget.videoDetail.videoGroupList.length, (index) {
                  return widget.videoDetail.videoGroupList[index].title;
                }).join(PunctuationMarks.comma)}',
                style: const TextStyle(
                  fontSize: FontSize.small,
                  color: CustomColors.gray,
                ),
              ),
            ),
            Container(
              padding: const EdgeInsets.only(bottom: 4),
              width: double.infinity,
              child: Text(
                '${Localization.getString(context, LocalizationStrings.createTime)} \u003A ${FormatParser.parseDateTime(widget.videoDetail.createTime)}',
                style: const TextStyle(
                  fontSize: FontSize.small,
                  color: CustomColors.gray,
                ),
              ),
            ),
            Container(
              padding: const EdgeInsets.only(bottom: 4),
              width: double.infinity,
              child: Text(
                '${Localization.getString(context, LocalizationStrings.director)} \u003A ${List.generate(widget.videoDetail.directorList.length, (index) {
                  return widget.videoDetail.directorList[index].name;
                }).join(PunctuationMarks.comma)}',
                style: const TextStyle(
                  fontSize: FontSize.small,
                  color: CustomColors.gray,
                ),
              ),
            ),
            Container(
              padding: const EdgeInsets.only(bottom: 4),
              width: double.infinity,
              child: Text(
                '${Localization.getString(context, LocalizationStrings.actor)} \u003A ${List.generate(widget.videoDetail.actorList.length, (index) {
                  return widget.videoDetail.actorList[index].name;
                }).join(PunctuationMarks.comma)}',
                style: const TextStyle(
                  fontSize: FontSize.small,
                  color: CustomColors.gray,
                ),
              ),
            ),
            Container(
              padding: const EdgeInsets.only(bottom: 4),
              width: double.infinity,
              child: Text(
                '${Localization.getString(context, LocalizationStrings.description)} \u003A ${widget.videoDetail.description}',
                style: const TextStyle(
                  fontSize: FontSize.small,
                  color: CustomColors.gray,
                ),
              ),
            ),
          ],
        ),
      ),
      isExpanded: _isExpanded,
    );
  }

  void _onExpansionCallback(int index, bool isExpanded) {
    setState(() {
      _isExpanded = !isExpanded;
    });
  }
}
