import 'package:flutter/material.dart';
import 'package:redinfo_integrated_ott_platform_flutter_sdk/constant/enum/communication_resource.dart';
import 'package:redinfo_integrated_ott_platform_flutter_sdk/delegate/communication_output_delegate.dart';
import 'package:redinfo_integrated_ott_platform_flutter_sdk/structure/entity/resource_state.dart';
import '../../constant/value/instance_container.dart';
import '../../constant/value/module_param.dart';
import '../../structure/entity/advertisement_image.dart';
import '../../util/instance_register.dart';
import '../presenter.dart';
import 'contract.dart';
import 'view.dart';

class MediaPlayerPresenter extends BasePresenter
    implements MediaPlayerPresenterContract {
  late int videoId;
  late int productCode;
  late String title;
  late String thumbnailPath;

  @override
  Widget createView(Map<String, dynamic> params) {
    videoId = params[ModuleParamKey.videoId] as int;
    productCode = 1234;
    title = params[ModuleParamKey.title] as String;
    thumbnailPath = params[ModuleParamKey.videoThumbnailPath] as String;
    return MediaPlayerView(
      presenter: this,
      url: params[ModuleParamKey.videoUrl] as String,
      advertisementImage:
          params[ModuleParamKey.advertisementImage] as AdvertisementImage,
      thumbnailPath: thumbnailPath,
    );
  }

  @override
  void initialViewData() {}

  @override
  Future<int> getMediaUsedResourceSize() {
    CommunicationOutputDelegate communicationOutputDelegate =
        InstanceRegister.resolve<CommunicationOutputDelegate>(
            InstanceContainer.communicationOutputDelegate);
    return communicationOutputDelegate.getUsedResourceSize(
        typeEnum: CommunicationResourceTypeEnum.movie);
  }

  @override
  Future<int> getMediaTotalResourceSize() {
    CommunicationOutputDelegate communicationOutputDelegate =
        InstanceRegister.resolve<CommunicationOutputDelegate>(
            InstanceContainer.communicationOutputDelegate);
    return communicationOutputDelegate.getTotalResourceSize(
        typeEnum: CommunicationResourceTypeEnum.movie);
  }

  @override
  Future<ResourceState> getMediaResourceState() {
    CommunicationOutputDelegate communicationOutputDelegate =
        InstanceRegister.resolve<CommunicationOutputDelegate>(
            InstanceContainer.communicationOutputDelegate);
    return communicationOutputDelegate.getResourceExistenceState(
      typeEnum: CommunicationResourceTypeEnum.movie,
      identity: "$videoId",
    );
  }

  @override
  Future<bool> onFireConsumeMedia() {
    CommunicationOutputDelegate communicationOutputDelegate =
        InstanceRegister.resolve<CommunicationOutputDelegate>(
            InstanceContainer.communicationOutputDelegate);
    return communicationOutputDelegate.consumeResource(
      typeEnum: CommunicationResourceTypeEnum.movie,
      identity: "$videoId",
      title: title,
      thumbnailPath: thumbnailPath,
    );
  }
}
