import 'package:flutter/material.dart';
import '../../bloc/relative_video_collection/bloc.dart';
import '../../bloc/relative_video_collection/delegate.dart';
import '../../bloc/relative_video_collection/event.dart';
import '../../bloc/relative_video_collection/state.dart';
import '../../constant/enum/route.dart';
import '../../constant/value/instance_container.dart';
import '../../constant/value/length.dart';
import '../../constant/value/module_param.dart';
import '../../delegate/route.dart';
import '../../structure/entity/relative_video_collection.dart';
import '../../structure/entity/video.dart';
import '../../util/instance_register.dart';
import '../presenter.dart';
import '../video_collection/contract.dart';
import 'contract.dart';
import 'view.dart';

class MediaRelativeVideoCollectionPresenter extends BasePresenter
    implements
        MediaRelativeVideoCollectionPresenterContract,
        VideoCollectionModuleDelegate {
  static const int offsetVideoLength = ListLength.small;
  MediaRelativeVideoCollectionModuleDelegate? delegate;

  @override
  Widget createView(Map<String, dynamic> params) {
    delegate = params[ModuleParamKey.moduleDelegate]
        as MediaRelativeVideoCollectionModuleDelegate?;
    return MediaRelativeVideoCollectionView(
      presenter: this,
      relativeVideoCollection: params[ModuleParamKey.relativeVideoCollection]
          as RelativeVideoCollection,
    );
  }

  @override
  void initialViewData() {}

  @override
  Widget getVideoCollectionWidget(List<Video> videoList) {
    RouteDelegate routeDelegate = InstanceRegister.resolve<RouteDelegate>(
        InstanceContainer.routeDelegate);
    return routeDelegate.getViewModule(ModuleViewKey.videoCollection, {
      ModuleParamKey.moduleDelegate: this,
      ModuleParamKey.crossAxisCount: 3,
      ModuleParamKey.enableScroll: false,
      ModuleParamKey.videoList: videoList,
    });
  }

  @override
  void onFireTapVideoEvent(int id) {
    delegate?.onFireTapVideoEvent(id);
  }
}
