import 'package:flutter/material.dart';
import 'package:movie_flutter/constant/value/animation.dart';
import '../../component/view/transition_wrapper.dart';
import '../../constant/enum/transition.dart';
import '../../constant/value/color.dart';
import '../../constant/value/length.dart';
import '../../constant/value/localization.dart';
import '../../constant/value/size.dart';
import '../../structure/entity/relative_video_collection.dart';
import '../../structure/entity/video.dart';
import '../../util/localization.dart';
import 'contract.dart';

class MediaRelativeVideoCollectionView extends StatefulWidget {
  final MediaRelativeVideoCollectionPresenterContract presenter;
  final RelativeVideoCollection relativeVideoCollection;

  const MediaRelativeVideoCollectionView({
    super.key,
    required this.presenter,
    required this.relativeVideoCollection,
  });

  @override
  State<MediaRelativeVideoCollectionView> createState() =>
      _MediaRelativeVideoCollectionViewState();
}

class _MediaRelativeVideoCollectionViewState
    extends State<MediaRelativeVideoCollectionView> {
  static const int _videoLength = ListLength.small;
  bool _onTransition = false;
  int _currentIndex = 0;
  int _groupVideoOffsetIndex = 0;
  int _actorVideoOffsetIndex = 0;

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        _getHeader(context),
        _getVideoCollectionWidget(),
      ],
    );
  }

  Widget _getHeader(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.symmetric(horizontal: 12),
      child: IntrinsicHeight(
        child: Row(
          children: [
            Expanded(
              child: Row(
                children: [
                  GestureDetector(
                    onTap: () => _onFireChangeIndex(0),
                    child: Text(
                      Localization.getString(
                          context, LocalizationStrings.groupSort),
                      style: TextStyle(
                        fontSize: FontSize.medium,
                        color: _currentIndex == 0
                            ? CustomColors.white
                            : CustomColors.gray,
                      ),
                    ),
                  ),
                  const VerticalDivider(
                    color: CustomColors.gray,
                    width: 8,
                    indent: 4,
                    endIndent: 6,
                  ),
                  GestureDetector(
                    onTap: () => _onFireChangeIndex(1),
                    child: Text(
                      Localization.getString(
                          context, LocalizationStrings.actorSort),
                      style: TextStyle(
                        fontSize: FontSize.medium,
                        color: _currentIndex != 0
                            ? CustomColors.white
                            : CustomColors.gray,
                      ),
                    ),
                  ),
                ],
              ),
            ),
            GestureDetector(
              onTap: _onFireOffsetEvent,
              child: Row(
                children: [
                  Text(
                    Localization.getString(context, LocalizationStrings.change),
                    style: const TextStyle(
                      fontSize: FontSize.medium,
                      color: CustomColors.amber,
                    ),
                  ),
                  const Icon(
                    Icons.cached_rounded,
                    color: CustomColors.amber,
                    size: IconSize.small,
                  ),
                ],
              ),
            ),
          ],
        ),
      ),
    );
  }

  Widget _getVideoCollectionWidget() {
    List<Video> originVideoList = (_currentIndex == 0)
        ? widget.relativeVideoCollection.groupVideoList
        : widget.relativeVideoCollection.actorVideoList;
    if (originVideoList.isEmpty) {
      return const SizedBox.shrink();
    }
    int offset =
        (_currentIndex == 0) ? _groupVideoOffsetIndex : _actorVideoOffsetIndex;
    List<Video> videoList = [];
    for (var i = 0; i < _videoLength; i++) {
      final videoIndex = (offset + i) % originVideoList.length;
      videoList.add(originVideoList[videoIndex]);
    }
    return TransitionWrapper(
      transitionKey:
          _onTransition ? TransitionKey.fadeOut : TransitionKey.fadeIn,
      duration: AnimationDuration.short,
      child: widget.presenter.getVideoCollectionWidget(videoList),
    );
  }

  void _onFireOffsetEvent() {
    if (_currentIndex == 0) {
      if (widget.relativeVideoCollection.groupVideoList.length <=
          _videoLength) {
        return;
      }
      setState(() {
        _onTransition = true;
        _groupVideoOffsetIndex += _videoLength;
        if (_groupVideoOffsetIndex >=
            widget.relativeVideoCollection.groupVideoList.length) {
          _groupVideoOffsetIndex -=
              widget.relativeVideoCollection.groupVideoList.length;
        }
      });
      Future.delayed(AnimationDuration.short).then((value) {
        setState(() {
          _onTransition = false;
        });
      });
    } else {
      if (widget.relativeVideoCollection.actorVideoList.length <=
          _videoLength) {
        return;
      }
      setState(() {
        _onTransition = true;
        _actorVideoOffsetIndex += _videoLength;
        if (_actorVideoOffsetIndex >=
            widget.relativeVideoCollection.actorVideoList.length) {
          _actorVideoOffsetIndex -=
              widget.relativeVideoCollection.actorVideoList.length;
        }
      });
      Future.delayed(AnimationDuration.short).then((value) {
        setState(() {
          _onTransition = false;
        });
      });
    }
  }

  void _onFireChangeIndex(int index) {
    if (_currentIndex != index) {
      setState(() {
        _onTransition = true;
        _currentIndex = index;
      });
      Future.delayed(AnimationDuration.short).then((value) {
        setState(() {
          _onTransition = false;
        });
      });
    }
  }
}
