import 'package:flutter/material.dart';
import '../../component/view/base64_file_image_provider.dart';
import '../../constant/value/animation.dart';
import '../../constant/value/assets.dart';
import '../../constant/value/color.dart';
import '../../constant/value/image.dart';
import '../../constant/value/size.dart';
import '../../structure/entity/video.dart';
import '../../util/assets.dart';
import 'contract.dart';

class VideoCollectionView extends StatelessWidget {
  static const double itemRatio = 5 / 8;
  final VideoCollectionPresenterContract presenter;
  final int crossAxisCount;
  final List<Video> videoList;

  const VideoCollectionView({
    super.key,
    required this.presenter,
    required this.crossAxisCount,
    required this.videoList,
  });

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
      return AspectRatio(
        aspectRatio: itemRatio *
            crossAxisCount /
            (videoList.length / crossAxisCount).ceil(),
        child: GridView.builder(
          physics: const NeverScrollableScrollPhysics(),
          gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
            crossAxisCount: crossAxisCount,
            childAspectRatio: itemRatio,
          ),
          itemCount: videoList.length,
          itemBuilder: (BuildContext context, int index) {
            return _getItemWidget(
              context,
              videoList[index],
              presenter.onFireTapVideoEvent,
            );
          },
        ),
      );
    });
  }
}

class ScrollableVideoCollectionView extends SliverGrid {
  static const double itemRatio = 99 / 160;
  final VideoCollectionPresenterContract presenter;
  final int crossAxisCount;
  final List<Video> videoList;

  ScrollableVideoCollectionView({
    super.key,
    required this.presenter,
    required this.crossAxisCount,
    required this.videoList,
  }) : super(
          gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
            crossAxisCount: crossAxisCount,
            childAspectRatio: itemRatio,
          ),
          delegate: SliverChildBuilderDelegate(
            (BuildContext context, int index) {
              return _getItemWidget(
                  context, videoList[index], presenter.onFireTapVideoEvent);
            },
            childCount: videoList.length,
          ),
        );
}

Widget _getItemWidget(
    BuildContext context, Video video, void Function(int id) onTap) {
  return Padding(
    padding: const EdgeInsets.only(top: 4.0, left: 4.0, right: 4.0),
    child: GestureDetector(
      onTap: () => onTap(video.id),
      child: Column(
        children: [
          ClipRRect(
            borderRadius: BorderRadius.circular(12),
            child: AspectRatio(
              aspectRatio: ImageAspectRatio.ratio_3_4,
              child: FadeInImage(
                fadeInDuration: AnimationDuration.tiny,
                fadeOutDuration: AnimationDuration.tiny,
                placeholder: AssetImage(
                    Assets.getAssetsPath(AssetsPath.imagePlaceHolder34)),
                image: Base64FileImageProvider(
                  video.thumbnailPath,
                ),
                fit: BoxFit.contain,
                width: double.infinity,
                height: double.infinity,
              ),
            ),
          ),
          Padding(
            padding: const EdgeInsets.only(top: 5.0, left: 4.0, right: 4.0),
            child: Text(
              video.title,
              maxLines: 1,
              overflow: TextOverflow.ellipsis,
              style: const TextStyle(
                fontSize: FontSize.medium,
                color: CustomColors.white,
              ),
            ),
          ),
        ],
      ),
    ),
  );
}
