import 'dart:async';
import 'package:dio/dio.dart';
import '../constant/value/network.dart';
import 'headers/index.dart' as headers_engine;

class NetworkHttpClient {
  final Map<String, String> headers = headers_engine.createHeaders();

  NetworkHttpClient({required Map<String, String> customHeaders}) {
    headers.addAll(customHeaders);
  }

  String buildUrl(String url, List<String> pathList) {
    String finalUrl = url;
    for (String path in pathList) {
      if (!path.startsWith("/")) {
        finalUrl += NetworkChar.slash;
      }
      finalUrl += path;
    }
    return finalUrl;
  }

  Future<String> get(String url, Map<String, String> queryParameters) async {
    Completer<String> completer = Completer();
    Dio()
        .get<String>(url,
            options: Options(headers: headers),
            queryParameters: queryParameters)
        .then((response) {
      if (response.statusCode != null && response.statusCode! < 400) {
        completer.complete(response.data);
      } else {
        completer.completeError("undefined error");
      }
    }).onError((error, stackTrace) {
      completer.completeError(error ?? "undefined error");
    });
    return completer.future;
  }

  Future<String> postFormData(String url, Map<String, String> textFields,
      Map<String, String> fileFields) async {
    Completer<String> completer = Completer();
    Map<String, dynamic> fields = {};
    textFields.forEach((key, value) {
      fields[key] = value;
    });
    for (var entry in fileFields.entries) {
      fields[entry.key] = await MultipartFile.fromFile(entry.value);
    }
    final formData = FormData.fromMap(fields);
    Dio()
        .post<String>(url, options: Options(headers: headers), data: formData)
        .then((response) {
      if (response.statusCode != null && response.statusCode! < 400) {
        completer.complete(response.data);
      } else {
        completer.completeError("undefined error");
      }
    }).onError((error, stackTrace) {
      completer.completeError(error ?? "undefined error");
    });
    return completer.future;
  }

  Future<String> putFormData(String url, Map<String, String> textFields,
      Map<String, String> fileFields) async {
    Completer<String> completer = Completer();
    Map<String, dynamic> fields = {};
    textFields.forEach((key, value) {
      fields[key] = value;
    });
    for (var entry in fileFields.entries) {
      fields[entry.key] = await MultipartFile.fromFile(entry.value);
    }
    final formData = FormData.fromMap(fields);
    Dio()
        .put<String>(url, options: Options(headers: headers), data: formData)
        .then((response) {
      if (response.statusCode != null && response.statusCode! < 400) {
        completer.complete(response.data);
      } else {
        completer.completeError("undefined error");
      }
    }).onError((error, stackTrace) {
      completer.completeError(error ?? "undefined error");
    });
    return completer.future;
  }
}
