import 'dart:async';
import 'dart:convert';

import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';
import 'package:flutter_localization/flutter_localization.dart';

class Localization {
  static String? _prefix;

  static void setUpPrefix(String prefix) {
    _prefix = prefix;
  }

  static Future<Map<String, dynamic>> mapFromJsonFile(
      {required String assets, String? prefix}) async {
    Completer<Map<String, dynamic>> completer = Completer();
    rootBundle.loadString(assets).then((value) {
      if (prefix == null) {
        completer.complete(jsonDecode(value));
      } else {
        Map<String, dynamic> newMap = {};
        for (var entry in jsonDecode(value).entries) {
          newMap["${prefix}_${entry.key}"] = entry.value;
        }
        completer.complete(newMap);
      }
    });
    return completer.future;
  }

  static String getString(BuildContext context, String text) {
    if (_prefix == null) {
      return text.getString(context);
    }
    return "${_prefix}_$text".getString(context);
  }
}
